import java.awt.*;
import java.awt.geom.*;
import java.lang.Math;
import java.text.NumberFormat;

/** Rectangulo en 2D. Subclase de Forma.*/

public class Rectangulo extends Forma
{
/** Las esquinas del rectangulo. Arreglo de cuatro puntos flotantes 2D.*/
	private Point2D[] esquina;

/** Constructor. Instancia un nuevo cuadrado paralelo a los ejes. Cada punto es una esquina arbitraria.
*@param a Esquina del rectangulo. Punto entero.
*@param b Esquina del rectangulo. Punto entero.*/

	Rectangulo (Point a, Point b)
	{
		double ax = a.getX();
		double ay = a.getY();
		double bx = b.getX();
		double by = b.getY();
		double temp;
		if (ax > bx)
		{
			temp = ax;
			ax = bx;
			bx = temp;
		}
		if (ay > by)
		{
			temp = ay;
			ay = by;
			by = temp;
		}
		esquina = new Point2D[4];
      esquina[0] = new Point2D.Double(ax, ay);
		esquina[1] = new Point2D.Double(ax, by);
		esquina[2] = new Point2D.Double(bx, by);
		esquina[3] = new Point2D.Double(bx, ay);
	}

/** Escala la forma respecto al punto central en un factor dado.
* @param c Punto respecto del cual se hace el escalamiento.
* @param factor Factor usado en el escalamiento. >1 corresponde a amplificacion, <1 corresponde a reduccion.*/

	public void escalar (Point c, double factor)
	{

		for (int i = 0; i < esquina.length; i++)
		{
			double x = c.getX() + factor*(esquina[i].getX() - c.getX());
			double y = c.getY() + factor*(esquina[i].getY() - c.getY());
			esquina[i].setLocation (x, y);
		}
	}

/** Rota la forma respecto al punto central en un angulo dado.
* @param c Punto respecto del cual se hace la rotacion.
* @param deltaAngulo Angulo de rotacion. En radianes.*/

	public void rotar (Point c, double deltaAngulo)
	{
		for (int i = 0; i < esquina.length; i++)
		{
			double x = esquina[i].getX() - c.getX();
			double y = esquina[i].getY() - c.getY();
			double distancia = Math.sqrt (x*x + y*y);
			double anguloInicial = Math.atan2 (y, x);
			x = distancia*Math.cos (anguloInicial + deltaAngulo) + c.getX();
			y = distancia*Math.sin (anguloInicial + deltaAngulo) + c.getY();
			esquina[i].setLocation (x, y);
		}	
	}

/** Traslada la forma, desplazandola en sentido X e Y.
* @param deltaX Desplazamiento en sentido horizontal.
* @param deltaY Desplazamiento en sentido vertical.*/

	public void trasladar (double deltaX, double deltaY)
	{
		for (int i = 0; i < esquina.length; i++)
			esquina[i].setLocation (esquina[i].getX() + deltaX, esquina[i].getY() + deltaY); 
	}

/** Retorna la posicion x del centro del rectangulo.
*@return Posicion x del centro. Doble.*/

	private double getCentroX ()
	{
		return (esquina[0].getX() + (esquina[2].getX() - esquina[0].getX())/2);
	}
	
/** Retorna la posicion y del centro del rectangulo.
*@return Posicion y del centro. Doble.*/

	private double getCentroY ()
	{
		return (esquina[0].getY() + (esquina[2].getY() - esquina[0].getY())/2);
	}
	
/** Retorna el alto del rectangulo.
*@return Alto del rectangulo. Doble.*/
	
	private double getAlto ()
	{
		double x = esquina[0].getX() - esquina[1].getX();
		double y = esquina[0].getY() - esquina[1].getY();
		return (Math.sqrt (x*x + y*y));
	}

/** Retorna el ancho del rectangulo.
*@return Ancho del rectangulo. Doble.*/

	private double getAncho ()
	{
		double x = esquina[0].getX() - esquina[3].getX();
		double y = esquina[0].getY() - esquina[3].getY();
		return (Math.sqrt (x*x + y*y));
	}

/** Retorna el angulo de inclinacion del rectangulo. En radianes.
*@return Angulo de inclinacion del rectangulo. Doble, en radianes.*/

	private double getAngulo ()
	{
		double x = esquina[0].getX() - esquina[3].getX();
		double y = esquina[0].getY() - esquina[3].getY();
		return (Math.toDegrees(Math.atan2 (y, x)));
	}

/** Retorna un string que representa al rectangulo. Los dobles son formateados a un maximo de tres decimales.
*@return String descriptivo. "Rectangulo: Centro; Alto, Ancho; Inclinacion".*/

	public String toString ()
	{
		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(3);
		return ("Rectangulo: Centro = (" +nf.format(getCentroX()) + ", " +nf.format(getCentroY()) + "); Alto = " +nf.format(getAlto()) + "; Ancho = " +nf.format(getAncho()) + "; Inclinacion = " + nf.format(getAngulo()) + ".");
	}

/** Verifica si este rectangulo es igual a otro objeto dado.
*@param otherObject Otro objeto a comparar.
*@return Verdadero si son iguales, falso de lo contrario.*/

	public boolean equals (Object otherObject)
	{
		if (this == otherObject) return true;
		if (otherObject == null) return false;
		if (getClass() != otherObject.getClass()) return false;
		Rectangulo other = (Rectangulo)otherObject;
		for (int i = 0; i < esquina.length; i++)
			if (!esquina[i].equals(other.esquina[i]))
				return false;
		return true;
	}

/** Clona este rectangulo.
*@return Un objecto identico a este rectangulo.*/

	public Object clone ()
	{
		Point clonA = new Point ((int)(getCentroX() - getAncho()/2), (int)(getCentroY() - getAlto()/2));
		Point clonB = new Point ((int)(getCentroX() + getAncho()/2), (int)(getCentroY() + getAlto()/2));
		Point centro = new Point ((int)getCentroX(), (int)getCentroY());
		Rectangulo clon = new Rectangulo (clonA, clonB);
		clon.rotar (centro, getAngulo());
		return clon;
	}
}